#include <stdbool.h>
#include <stdint.h>

#include "gdt.h"
#include "idt.h"

extern void *isr_stub_table[];

__attribute__((aligned(0x10))) static struct idt_entry idt_entries[IDT_SIZE];
static struct idt_descriptor idtr;
/*
static void set_idt_entry_value(uint16_t *target, uint32_t offset,
                                uint16_t selector, uint8_t dpl,
                                uint8_t gate_type)
{
        // Encode the offset
        target[0] = offset & 0xFFFF;         // Low 16 bits
        target[3] = (offset >> 16) & 0xFFFF; // High 16 bits

        // Encode the presence (bit 15)
        target[1] |= 1 << 15;

        // Encode the CPU Privilege Levels (DPL)
        target[1] &= ~(0b11 << 13);      // Clear previous DPL
        target[1] |= (dpl & 0b11) << 13; // Set new DPL

        // Clear bit 12 (always 0 for interrupts)
        target[1] &= ~(1 << 12);

        // Encode Gate Type
        target[1] &= ~0x0F00;                 // Clear previous gate type
        target[1] |= (gate_type & 0x0F) << 8; // Set new gate type

        // Encode selector
        target[2] = selector;
}*/

void idt_set_descriptor(uint8_t index, void *isr, uint8_t flags)
{
	struct idt_entry *descriptor = &idt_entries[index];

	descriptor->isr_low = (uint32_t)isr & 0xFFFF;
	descriptor->kernel_cs = GDT_OFFSET_KERNEL_CODE;
	descriptor->attributes = flags;
	descriptor->isr_high = (uint32_t)isr >> 16;
	descriptor->reserved = 0;
}

void init_idt(void)
{
	idtr.offset = (uintptr_t)&idt_entries[0];
	idtr.size = (uint16_t)sizeof(struct idt_entry) * IDT_SIZE - 1;

	for (uint8_t i = 0; i < 32; i++)
		idt_set_descriptor(i, isr_stub_table[i], 0x8E);

	__asm__ volatile("lidt %0" : : "m"(idtr));
	__asm__ volatile("sti");
}
